@@ -67,10 +67,10 @@ def cut_spectrum(minWavelength, maxWavelength, fixedMirrorData, movingMirrorData
     return fixedMirrorData, movingMirrorData, wavelengths, tdsiTrace
 
 
-def substractNoiseFloor(fixedMirrorData, movingMirrorData):
+def substractNoiseFloor(fixedMirrorData, movingMirrorData,relativeNoiseFloor):
     """ Substract noise floor from spectra (except trace) """
     
-    lowLimit = 0.01
+    lowLimit = relativeNoiseFloor
     
     fixedMirrorData = fixedMirrorData - np.max((fixedMirrorData[0],fixedMirrorData[-1]))
     fixedMirrorData[fixedMirrorData<0] = 0
@@ -115,72 +115,23 @@ def find_shear(wavelengths, upconvPowerSpectrum, movingMirrorData, movingMirror_
     df = np.max( np.diff(frequencies) ) / 16
     N = int(round(Df / df))
     linFreqs = np.linspace(frequencies[-1]-(N-1)*df, frequencies[-1], N )
-    upconvPowerSpectrum = np.interp(linFreqs, frequencies, upconvPowerSpectrum)
+    upconvPowerSpectrum = np.interp(linFreqs, frequencies, upconvPowerSpectrum*C/frequencies**2)
     
     
     newMovingMirrorData = np.zeros((movingMirrorData.shape[0], linFreqs.shape[0]))
     
     for ii in range( movingMirrorData.shape[0] ):
-        newMovingMirrorData[ii,:] = np.interp(linFreqs, frequencies, movingMirrorData[ii,:])
+        newMovingMirrorData[ii,:] = np.interp(linFreqs, frequencies, movingMirrorData[ii,:]*C/frequencies**2)
     
     movingMirrorData = newMovingMirrorData
     frequencies = linFreqs
     
-    
-    
-    #crossCorr = np.zeros( (movingMirrorData.shape[0], movingMirrorData.shape[1]))
-    crossCorr = np.zeros( (movingMirrorData.shape[0], movingMirrorData.shape[1]*2-1))
-
-    lags = np.zeros_like(crossCorr)
     shearMap = np.zeros( movingMirrorData.shape[0] )
+    v0 = np.trapz(upconvPowerSpectrum*linFreqs,linFreqs) / np.trapz(upconvPowerSpectrum,linFreqs)
     
     for ii in range( movingMirrorData.shape[0] ):
-        #lags[ii,:], crossCorr[ii,:] =fq.ezcorr(frequencies, movingMirrorData[ii,:], upconvPowerSpectrum) 
-        #shearMap[ii] = lags[ii,:][ crossCorr[ii,:] == np.max(crossCorr[ii,:]) ]
-        crossCorr[ii,:] =  np.correlate(movingMirrorData[ii,:], upconvPowerSpectrum,'full')
-        maxId = np.argmax(crossCorr[ii,:])
-        peakFreq = -(N - (maxId+1))*df
-        lags = -(N - np.linspace(1,2*N-1,2*N-1) ) *df
-        
-        
-        x,y = fq.ezdiff(lags, crossCorr[ii,:])
-        
-        f = interp.interp1d(x, y, kind = 'cubic')
-        
-        err = 1
-        threshold = 1e-5
-        maxIter = 1000
-        nIter = 0
-        x0 = peakFreq - 5*df
-        x1 = peakFreq + 5*df
-            
-        while err > threshold:
-            nIter += 1
-            if nIter > maxIter:
-                break
-                
-            if x0<np.min(x):
-                x0 = np.min(x)
-            if x0>np.max(x):
-                x0 = np.max(x)
-                
-            if x1<np.min(x):
-                x1 = np.min(x)
-            if x1>np.max(x):
-                x1 = np.max(x)
-                    
-            f0 = f(x0)
-            f1 = f(x1)
-            dfdx = (f1-f0) / (x1 - x0)
-            b = f0 - dfdx*x0
-            
-            x0 = x1
-            x1 = -b/dfdx
-            err = abs((x1-x0)/x0)
-            
-             
-        
-        shearMap[ii] = x1# -(N - (maxId+1))*df
+        shearMap[ii] = np.trapz(movingMirrorData[ii,:]*linFreqs,linFreqs) / np.trapz(movingMirrorData[ii,:],linFreqs) - v0
+
     
     p = np.polyfit(movingMirror_Z, shearMap, 1)
     
@@ -188,18 +139,12 @@ def find_shear(wavelengths, upconvPowerSpectrum, movingMirrorData, movingMirror_
         
     if debugGraphs:
         plt.figure()
-        plt.plot(movingMirror_Z,shearMap/1e12, LineWidth = 2, label = 'Data')
-        plt.plot(movingMirror_Z, np.polyval(p, movingMirror_Z) /1e12, 'r', LineWidth = 2, label = 'Linear fit')
+        plt.plot(movingMirror_Z,shearMap/1e12,'b', linewidth = 2, label = 'Data')
+        plt.plot(movingMirror_Z, np.polyval(p, movingMirror_Z) /1e12, '--r', linewidth = 2, label = 'Linear fit')
         plt.legend()
         plt.xlabel(r'Stage displacement [$\mu$m]')
         plt.ylabel('Shear [THz]')
         
-        plt.figure()
-        plt.pcolor(lags/1e12, movingMirror_Z, crossCorr)
-        plt.plot(shearMap/1e12, movingMirror_Z, '--r')
-        plt.xlim(-20,20)
-        plt.xlabel('Shear frequency [THz]')
-        plt.ylabel(r'Stage displacement [$\mu$m]')
     
     return shear
 
@@ -233,65 +178,24 @@ def find_upconversion(wavelengths, upconvPowerSpectrum, fundWavelengths, fundPow
     N = int(round((maxFreq -minFreq) / Df))
     linFreqs = np.linspace(maxFreq, minFreq, N )
     
-    upconvPowerSpectrum = np.interp(linFreqs, frequencies, upconvPowerSpectrum)
-    fundPowerSpectrum = np.interp(linFreqs, fundFrequencies, fundPowerSpectrum)
+    upconvPowerSpectrum = np.interp(linFreqs, frequencies, upconvPowerSpectrum*C/frequencies**2)
+    fundPowerSpectrum = np.interp(linFreqs, fundFrequencies, fundPowerSpectrum*C/fundFrequencies**2)
     
     frequencies = linFreqs
     
+    v0Fund = np.trapz(fundPowerSpectrum*linFreqs,linFreqs) / np.trapz(fundPowerSpectrum,linFreqs)
+    v0Upconv = np.trapz(upconvPowerSpectrum*linFreqs,linFreqs) / np.trapz(upconvPowerSpectrum,linFreqs)
+    
+    upconvFrequency = v0Upconv - v0Fund
+    upconvWavelength = C / upconvFrequency
+    
+    newWav = C/(linFreqs-upconvFrequency)
+    
+    downConvWavelengths = np.linspace(fundWavelengths.min(),fundWavelengths.max(),fundWavelengths.shape[0])
+    downConvSpectrum =  np.interp(fundWavelengths,newWav[newWav>0] , upconvPowerSpectrum[newWav>0]*(linFreqs-upconvFrequency)[newWav>0]**2/C)
     
     
-    #crossCorr = np.zeros( (movingMirrorData.shape[0], movingMirrorData.shape[1]))
-    crossCorr = np.zeros( linFreqs.shape[0]*2-1 )
-
-    lags = np.zeros_like(crossCorr)
-    
-    crossCorr =  np.correlate(fundPowerSpectrum, upconvPowerSpectrum,'full')
-    maxId = np.argmax(crossCorr)
-    peakFreq = -(N - (maxId+1))*Df
-    lags = -(N - np.linspace(1,2*N-1,2*N-1) ) *Df
-    
-    
-    x,y = fq.ezdiff(lags, crossCorr)
-    
-    f = interp.interp1d(x, y, kind = 'cubic')
-    
-    err = 1
-    threshold = 1e-5
-    maxIter = 1000
-    nIter = 0
-    x0 = peakFreq - 5*Df
-    x1 = peakFreq + 5*Df
-        
-    while err > threshold:
-        nIter += 1
-        if nIter > maxIter:
-            break
-            
-        if x0<np.min(x):
-            x0 = np.min(x)
-        if x0>np.max(x):
-            x0 = np.max(x)
-            
-        if x1<np.min(x):
-            x1 = np.min(x)
-        if x1>np.max(x):
-            x1 = np.max(x)
-                
-        f0 = f(x0)
-        f1 = f(x1)
-        dfdx = (f1-f0) / (x1 - x0)
-        b = f0 - dfdx*x0
-        
-        x0 = x1
-        x1 = -b/dfdx
-        err = abs((x1-x0)/x0)
-        
-         
-    
-    upconvWavelength = C/x1# -(N - (maxId+1))*df
-    
-    
-    return upconvWavelength
+    return upconvWavelength, downConvWavelengths, downConvSpectrum
 
 def make1Dfft(wavelengths,stagePosition,trace,zeropadding = True, windowing = True, debugGraphs= False):
 
@@ -299,11 +203,13 @@ def make1Dfft(wavelengths,stagePosition,trace,zeropadding = True, windowing = Tr
     
     dz = np.diff(stagePosition)[0]
     
+    zMin = stagePosition[0]
+    zMax = stagePosition[-1]
     
     plt.figure()
-    plt.pcolor(wavelengths*1e9,stagePosition,np.abs(trace)/np.max(np.abs(trace)))
+    plt.pcolormesh(wavelengths*1e9,(stagePosition-(zMin+zMax)/2)*1e-6*2/C*1e15,np.abs(trace)/np.max(np.abs(trace)),shading='auto')
     plt.xlabel(r'Wavelengths [nm]')
-    plt.ylabel(r'Stage displacement [$\mu$m]')
+    plt.ylabel(r'Delay [fs]')
     c = plt.colorbar()
     c.set_label('Relative intensity')
     
@@ -350,9 +256,9 @@ def make1Dfft(wavelengths,stagePosition,trace,zeropadding = True, windowing = Tr
     
     if debugGraphs:
         plt.figure()
-        plt.pcolor(wavelengths*1e9, spatialFreqs, absfTrace/np.max(absfTrace))
+        plt.pcolormesh(wavelengths*1e9, spatialFreqs/1e-6/2*C/1e12, absfTrace/np.max(absfTrace),shading='auto')
         plt.xlabel('Wavelengths [nm]')
-        plt.ylabel(r'Spatial frequency [$\mu$m$^{-1}$]')
+        plt.ylabel(r'Frequency [THz]')
         c = plt.colorbar()
         c.set_label('Relative intensity')
     
@@ -361,7 +267,7 @@ def make1Dfft(wavelengths,stagePosition,trace,zeropadding = True, windowing = Tr
                
 def cut_fft(amplitude, phase, wavelengths,debugGraphs):
     
-    cut = 0.01
+    cut = 0.00
     extend = 0
     
     Icut = amplitude >= cut*np.max(amplitude)
@@ -402,21 +308,21 @@ def cut_fft(amplitude, phase, wavelengths,debugGraphs):
                
                
 
-def calc_spectral_phase(shearFrequency, upconvWavelength,FFTamplitude, FFTphase, FFTwavelengths,polyfit,debugGraphs):
+def calc_spectral_phase(shearFrequency, upconvWavelength,FFTamplitude, FFTphase, FFTwavelengths,debugGraphs):
     
     
     shearW = 2*np.pi * shearFrequency
     
     phase  = -FFTphase # Minus to "correct GVD sign"...
     
-    concW, concPhase,concGD, concGDD, concTOD = phase_concatenation(FFTwavelengths,FFTamplitude, shearW, phase, upconvWavelength,polyfit=polyfit ,debugGraphs =debugGraphs)
-    midpointW, midpointPhase, midpointGD, midpointGDD, midpointTOD = phase_midpoint(FFTwavelengths,FFTamplitude, shearW, phase, upconvWavelength,polyfit=polyfit ,debugGraphs =debugGraphs)
+    concW, concPhase = phase_concatenation(FFTwavelengths,FFTamplitude, shearW, phase, upconvWavelength ,debugGraphs =debugGraphs)
+    midpointW, midpointPhase = phase_midpoint(FFTwavelengths,FFTamplitude, shearW, phase, upconvWavelength ,debugGraphs =debugGraphs)
     
-    return concW, concPhase,concGD, concGDD, concTOD, midpointW, midpointPhase,midpointGD, midpointGDD, midpointTOD
+    return concW, concPhase, midpointW, midpointPhase
     
     
     
-def phase_concatenation(wavelengths,amplitude, shearW, phase, upconvWavelength, polyfit = 0,debugGraphs= False):
+def phase_concatenation(wavelengths,amplitude, shearW, phase, upconvWavelength,debugGraphs= False):
     
     wCut = 2*np.pi * C *( 1/wavelengths - 1/upconvWavelength)
     
@@ -450,9 +356,6 @@ def phase_concatenation(wavelengths,amplitude, shearW, phase, upconvWavelength,
     if shearW < 0:
         concPhase = -concPhase
     
-    concGD = np.zeros_like(concPhase[:,1:-1])
-    concGDD = np.zeros_like(concPhase[:,1:-1])
-    concTOD = np.zeros_like(concPhase[:,2:-2])
     
     for ii in range(shearMultiplicity):
         
@@ -462,13 +365,6 @@ def phase_concatenation(wavelengths,amplitude, shearW, phase, upconvWavelength,
         
         concPhase[ii,:] = concPhase[ii,:] - np.polyval(linFit, concW[ii,:])
     
-        if polyfit >= 2:
-            phaseFit =  np.polyfit(concW[ii,:], concPhase[ii,:], polyfit, w = weights)
-            concPhase[ii,:] = np.polyval(phaseFit, concW[ii,:])
-        
-        concGD[ii,:] = fq.ezdiff(concW[ii,:], concPhase[ii,:], n=1, order = 2)[1]
-        concGDD[ii,:] = fq.ezdiff(concW[ii,:], concPhase[ii,:], n=2, order = 2)[1]
-        concTOD[ii,:] = fq.ezdiff(concW[ii,:], concPhase[ii,:], n=3, order = 2)[1]
     
     if debugGraphs:
         plt.figure()
@@ -476,10 +372,10 @@ def phase_concatenation(wavelengths,amplitude, shearW, phase, upconvWavelength,
         plt.ylabel('Spectral phase (concat. method) [rad]')
         plt.xlabel('Downconv. wavelength [nm]')
         
-    return concW, concPhase, concGD, concGDD, concTOD
+    return concW, concPhase
         
 
-def phase_midpoint(wavelengths,amplitude, shearW, phase, upconvWavelength, polyfit = 0,debugGraphs= False):
+def phase_midpoint(wavelengths,amplitude, shearW, phase, upconvWavelength,debugGraphs= False):
     
     wCut = 2*np.pi * C *( 1/wavelengths - 1/upconvWavelength)
     
@@ -503,29 +399,14 @@ def phase_midpoint(wavelengths,amplitude, shearW, phase, upconvWavelength, polyf
     deltaOfW = integrate.cumtrapz(phaseCutLin, wCutLin)
     midpointW = wCutLin[1:]
     
-    midpointPhase = -np.interp(midpointW-shearW/2, midpointW, deltaOfW) / shearW
+    midpointPhase = -interp.interp1d(midpointW, deltaOfW,'linear',bounds_error = False, fill_value = 'extrapolate')(midpointW+shearW/2) / shearW
    
     
-    
-    
-    
-    midpointGD = np.zeros_like(midpointPhase[1:-1])
-    midpointGDD = np.zeros_like(midpointPhase[1:-1])
-    midpointTOD = np.zeros_like(midpointPhase[2:-2])
-    
     weights = np.interp(midpointW, wCutLin, amplitudeCutLin)
     linFit = np.polyfit(midpointW, midpointPhase, 1, w = weights)
         
     midpointPhase = midpointPhase - np.polyval(linFit, midpointW)
     
-    if polyfit >= 2:
-        phaseFit =  np.polyfit(midpointW, midpointPhase, polyfit, w = weights)
-        midpointPhase = np.polyval(phaseFit, midpointW)
-    
-    
-    midpointGD = fq.ezdiff(midpointW, midpointPhase, n=1, order = 2)[1]
-    midpointGDD = fq.ezdiff(midpointW, midpointPhase, n=2, order = 2)[1]
-    midpointTOD = fq.ezdiff(midpointW, midpointPhase, n=3, order = 2)[1]
     
     if debugGraphs:
         plt.figure()
@@ -533,20 +414,22 @@ def phase_midpoint(wavelengths,amplitude, shearW, phase, upconvWavelength, polyf
         plt.ylabel('Spectral phase (midpoint method) [rad]')
         plt.xlabel('Downconv. wavelength [nm]')
         
-    return midpointW, midpointPhase,midpointGD, midpointGDD, midpointTOD
+    return midpointW, midpointPhase
 
 
 def calc_temporal_envelope(wavelengths, upconvPowerSpectrum, upconvWavelength, angFreqPhase, phase, plotSpectrum,minTimeResolution):
     
     pulse = []
+    eFieldOut = []
+    
+    frequency = np.flip( C * (1/wavelengths - 1/upconvWavelength) )
+    spectrumEnvelope = np.flip(  upconvPowerSpectrum* wavelengths**2/C/np.max(upconvPowerSpectrum* wavelengths**2/C)  )
     
     if len(phase.shape) > 1:
     
         for ii in range(phase.shape[0]):
         
-            frequency = np.flip( C * (1/wavelengths - 1/upconvWavelength) )
             
-            spectrumEnvelope = np.flip( np.sqrt(upconvPowerSpectrum/np.max(upconvPowerSpectrum)) )
             
             nuMax = np.max(frequency)
             nuMin = np.min(frequency)
@@ -563,37 +446,20 @@ def calc_temporal_envelope(wavelengths, upconvPowerSpectrum, upconvWavelength, a
             
             II = ( (nu>=nuMin) & (nu<=nuMax) )
             
-            interpEnvelope[II] = np.interp(nu[II], frequency, spectrumEnvelope)
+            interpEnvelope[II] = np.sqrt( np.interp(nu[II], frequency, spectrumEnvelope) )
             interpPhase[II] = np.interp(nu[II], angFreqPhase[ii,:]/(2*np.pi), phase[ii,:])
             interpPhase[nu<nuMin] = interpPhase[II][0]
             interpPhase[nu>nuMax] = interpPhase[II][-1]
             
             spectrum = interpEnvelope * np.exp(1j * interpPhase)
             
-            t, eField = fq.ezifft(nu, spectrum)
-            eField = np.fft.ifftshift(eField)
+            t, eField = fq.ezifft(nu, spectrum, amplitudeSpectrumRecentering = True)
             
+            eFieldOut.append( eField )
             pulse.append( np.abs(eField) )
-# =============================================================================
-#             if (plotSpectrum and ii==0):
-#                 IIplotphase = interpEnvelope[nu>0] > np.max(interpEnvelope)/10
-#                 
-#                 
-#                 fig = plt.figure()
-#                 axLeft = fig.gca()
-#                 axRight = axLeft.twinx()
-#                 axLeft.plot(C/nu[nu>0]*1e9, interpEnvelope[nu>0]**2)
-#                 axRight.plot(C/nu[nu>0][IIplotphase]*1e9,interpPhase[nu>0][IIplotphase], '--r')
-#                 axLeft.set_xlim(np.min(C/nu[nu>0]*1e9),1200)
-#                 axLeft.set_xlabel('Wavelength [nm]')
-#                 axLeft.set_ylabel('Normalized intensity', color = 'blue')
-#                 axRight.set_ylabel('Spectral phase [rad]', color = 'red')
-# =============================================================================
     
     else:
-        frequency = np.flip( C * (1/wavelengths - 1/upconvWavelength) )
             
-        spectrumEnvelope = np.flip( np.sqrt(upconvPowerSpectrum/np.max(upconvPowerSpectrum)) )
         
         nuMax = np.max(frequency)
         nuMin = np.min(frequency)
@@ -610,164 +476,32 @@ def calc_temporal_envelope(wavelengths, upconvPowerSpectrum, upconvWavelength, a
         
         II = ( (nu>=nuMin) & (nu<=nuMax) )
         
-        interpEnvelope[II] = np.interp(nu[II], frequency, spectrumEnvelope)
+        interpEnvelope[II] =np.sqrt(  np.interp(nu[II], frequency, spectrumEnvelope) )
         interpPhase[II] = np.interp(nu[II], angFreqPhase/(2*np.pi), phase)
         interpPhase[nu<nuMin] = interpPhase[II][0]
         interpPhase[nu>nuMax] = interpPhase[II][-1]
         
         spectrum = interpEnvelope * np.exp(1j * interpPhase)
         
-        t, eField = fq.ezifft(nu, spectrum)
-        eField = np.fft.ifftshift(eField)
+        t, eField = fq.ezifft(nu, spectrum, amplitudeSpectrumRecentering = True)
         
+        eFieldOut.append( eField )
         pulse.append( np.abs(eField) )
-# =============================================================================
-#         if (plotSpectrum):
-#             IIplotphase = interpEnvelope[nu>0] > np.max(interpEnvelope)/10
-#             
-#             
-#             fig = plt.figure()
-#             axLeft = fig.gca()
-#             axRight = axLeft.twinx()
-#             axLeft.plot(C/nu[nu>0]*1e9, interpEnvelope[nu>0]**2)
-#             axRight.plot(C/nu[nu>0][IIplotphase]*1e9,interpPhase[nu>0][IIplotphase], '--r')
-#             axLeft.set_xlim(np.min(C/nu[nu>0]*1e9),1200)
-#             axLeft.set_xlabel('Wavelength [nm]')
-#             axLeft.set_ylabel('Normalized intensity', color = 'blue')
-#             axRight.set_ylabel('Spectral phase [rad]', color = 'red')
-# =============================================================================
         
     
+    eFieldOut = np.array(eFieldOut)
+    eFieldOut = np.mean(eFieldOut, axis = 0)
+    
     pulse = np.array(pulse)
     pulse = np.mean(pulse, axis = 0)
     pulse = pulse**2 / np.max(pulse**2)
     
     
     
-    return t, pulse, eField
+    return t, pulse, eFieldOut
     
 
 
 
-def make_shear_map(wavelengths, upconvPowerSpectrum, movingMirrorData, movingMirror_Z,debugGraphs):
-    """ Calculate shear frequency as a function of stage position,
-    and take its value at the middle position as constant approximation """
-    
-    # Normalize spectra to max of one
-    upconvPowerSpectrum = upconvPowerSpectrum / np.max(upconvPowerSpectrum)
-    
-    maxValues = np.max(movingMirrorData, axis = 1)
-    tmp = wavelengths.shape[0]
-    maxValues = np.transpose( np.tile(maxValues, (tmp,1)) )
-    movingMirrorData = movingMirrorData / maxValues
-    
 
-
-    # Convert wavelengths to frequencies
-    frequencies = C / wavelengths
-    
-    frequencies = np.flip(frequencies) # Flipping from low to high frequencies
-    upconvPowerSpectrum = np.flip(upconvPowerSpectrum)
-    movingMirrorData = np.flip(movingMirrorData,axis = 1)
-    
-    # Interpolate to a linear spacing of frequencies
-    # Choice of datapoint position strongly affect results, here I am copying Matlab
-    # need to check if another strategy would work better
-    Df = frequencies[-1] - frequencies[0]
-    df = np.max( np.diff(frequencies) ) / 2 #Added this division by two as a test
-    N = round(Df / df)
-    linFreqs = np.linspace(frequencies[-1]-(N-1)*df, frequencies[-1], N )
-    upconvPowerSpectrum = np.interp(linFreqs, frequencies, upconvPowerSpectrum)
-    
-    
-    newMovingMirrorData = np.zeros((movingMirrorData.shape[0], linFreqs.shape[0]))
-    
-    for ii in range( movingMirrorData.shape[0] ):
-        newMovingMirrorData[ii,:] = np.interp(linFreqs, frequencies, movingMirrorData[ii,:])
-    
-    movingMirrorData = newMovingMirrorData
-    frequencies = linFreqs
-    
-    
-    
-    #crossCorr = np.zeros( (movingMirrorData.shape[0], movingMirrorData.shape[1]))
-    crossCorr = np.zeros( (movingMirrorData.shape[0], movingMirrorData.shape[1]*2-1))
-
-    lags = np.zeros_like(crossCorr)
-    shearMap = np.zeros( movingMirrorData.shape[0] )
-    
-    for ii in range( movingMirrorData.shape[0] ):
-        #lags[ii,:], crossCorr[ii,:] =fq.ezcorr(frequencies, movingMirrorData[ii,:], upconvPowerSpectrum) 
-        #shearMap[ii] = lags[ii,:][ crossCorr[ii,:] == np.max(crossCorr[ii,:]) ]
-        crossCorr[ii,:] =  np.correlate(movingMirrorData[ii,:], upconvPowerSpectrum,'full')
-        maxId = np.argmax(crossCorr[ii,:])
-        
-        lags = -(N - np.linspace(1,2*N-1,2*N-1) ) *df
-        peakFreq = np.average(lags, weights = crossCorr[ii,:])
-        
-        
-        shearMap[ii] = peakFreq#-(N - (maxId+1))*df
-    
-    p = np.polyfit(movingMirror_Z, shearMap, 1)
-    
-    shearMap = np.polyval(p, movingMirror_Z)
-        
-    if debugGraphs:
-        plt.figure()
-        plt.plot(movingMirror_Z/1000,shearMap/1e12, LineWidth = 2, label = 'Data')
-        plt.plot(movingMirror_Z/1000, np.polyval(p, movingMirror_Z) /1e12, 'r', LineWidth = 2, label = 'Linear fit')
-        plt.legend()
-        plt.xlabel(r'Stage displacement [$\mu$m]')
-        plt.ylabel('Shear [THz]')
-    
-    return shearMap
-
-
-def simFrogTrace(t,E,minWavelength,maxWavelength):
-    
-    I = np.abs(E)**2
-    
-    t0 = -200e-15#t[t<0][I[t<0]<np.max(I)/100][-1]*2
-    t1 = 200e-15#t[t>0][I[t>0]<np.max(I)/100][0]*2
-    
-    if abs(t1) > abs(t0):
-        t0 = -t1
-    else:
-        t1 = -t0
-    
-    lags = np.linspace(t0, t1, 400)
-    
-    wavelengths = np.linspace(minWavelength, maxWavelength, 1000)
-    frogTrace = np.zeros((wavelengths.shape[0],lags.shape[0]))
-    
-    for ii,lag in enumerate(lags):
-        
-        t2 = t + lag
-        E2 = np.interp(t,t2,E)
-        
-        Esfg = E * E2
-        
-        
-        nu, S = fq.ezfft(t, Esfg)
-        
-        wav = C/np.flip(nu)
-        S = np.flip(S)
-        spectrum = np.interp(wavelengths, wav, np.abs(S)**2)
-        
-        frogTrace[:,ii] = spectrum
-    
-    frogTrace = frogTrace / np.max(frogTrace)
-    
-    plt.figure()
-    plt.pcolor(lags*1e15, wavelengths*1e9, frogTrace, cmap = 'viridis')
-    plt.gca().invert_yaxis()
-    plt.xlabel('Delay [fs]')
-    plt.ylabel('Wavelength [nm]')
-    c = plt.colorbar()
-    c.set_label('Relative intensity')
-    
-    
-        
-        
-    
     
\ No newline at end of file
